import React from 'react';
import './ReplyMessagePreview.scss';
import { MessageEntity } from '../../../../Domain/entity/MessageEntity';
import ReplyFilled from '../../../components/UI/svgs/Icons/Actions/ReplyFilled';
import { FunctionTypeVoidToVoid } from '../../../../CommonTypes/BaseViewModel';
import Close from '../../../components/UI/svgs/Icons/Navigation/Close';
import { FileType } from '../../../../Domain/entity/FileTypes';
import ReplyImagePreviewAttachment from './ReplyImagePreviewAttachment/ReplyImagePreviewAttachment';
import TextDocument from '../../../components/UI/svgs/Icons/Media/TextDocument';

type ReplyMessagePreviewProps = {
  messages: MessageEntity[];
  userNameSentMessage: string;
  // userId: string;
  onClose: FunctionTypeVoidToVoid;
};

function trimMessage(messageText: string) {
  return `${messageText.substring(0, 39)} ...`;
}

// eslint-disable-next-line react/function-component-definition
const ReplyMessagePreview: React.FC<ReplyMessagePreviewProps> = ({
  messages,
  userNameSentMessage,
  onClose,
}: ReplyMessagePreviewProps) => {
  return (
    <div className="reply-message-preview-row">
      <div className="reply-message-preview-row-card">
        {messages.length > 0 &&
        messages[0].attachments &&
        messages[0].attachments.length > 0 ? (
          <div className="reply-message-preview-placeholder">
            <div className="reply-message-preview-placeholder-icon">
              {messages &&
              messages[0].attachments &&
              messages[0].attachments.length > 0 &&
              messages[0].attachments[0].type
                .toString()
                .includes(FileType.image) &&
              messages[0].attachments[0].file ? (
                <ReplyImagePreviewAttachment
                  imageFile={messages[0].attachments[0].file}
                />
              ) : (
                <TextDocument width="24" height="24" color="var(--caption)" />
              )}
            </div>
          </div>
        ) : null}

        <div className="reply-message-preview-message">
          <div className="reply-message-preview-message-caption">
            <div className="reply-message-preview-message-caption-info">
              <div className="reply-message-preview-message-caption-info-icon">
                <ReplyFilled width="16" height="16" color="var(--caption)" />
              </div>
              <div className="reply-message-preview-message-caption-info-replied-to-name">
                Reply to {userNameSentMessage}
              </div>
            </div>
          </div>
          {messages.length > 0 &&
          (!messages[0].attachments ||
            (messages[0].attachments &&
              messages[0].attachments.length === 0)) ? (
            <div className="reply-message-preview-message-caption-info-text">
              {messages.length > 1
                ? '2 messages'
                : trimMessage(messages[0].message)}
            </div>
          ) : (
            <div className="reply-message-preview-message-caption-info-file">
              {messages.length > 1
                ? '2 messages'
                : trimMessage(messages[0].message)}
            </div>
          )}
        </div>
      </div>
      <div
        className="reply-message-preview-message-navigation-close"
        onClick={onClose}
      >
        <Close width="24" height="24" color="var(--secondary-elements)" />
      </div>
    </div>
  );
};

export default ReplyMessagePreview;
